"""ConfigPig — Python SDK for managing configs via API."""

__version__ = "0.1.0"

from .client import Client
from .drop_in import configure, get_config
from .types import (
    APIResponse,
    Config,
    ConfigLabel,
    ConfigSummary,
    ConfigVersion,
    Export,
    FetchResult,
    HealthCheck,
    LabelInfo,
    TeamLabel,
)

__all__ = [
    "Client",
    "configure",
    "get_config",
    "APIResponse",
    "Config",
    "ConfigLabel",
    "ConfigSummary",
    "ConfigVersion",
    "Export",
    "FetchResult",
    "HealthCheck",
    "LabelInfo",
    "TeamLabel",
]
